@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <errno.h>
 
 #include <libexif/exif-ifd.h>
@@ -35,13 +36,15 @@
 
 #define TAG_VALUE_BUF 1024
 
+#define SEP " "
+
 static void
 convert_arg_to_entry (const char *set_value, ExifEntry *e, ExifByteOrder o, ExifLog *log)
 {
 	unsigned int i, numcomponents;
-	char *value_p;
+	char *value_p, *buf;
 
-        /*
+	 /*
 	 * ASCII strings are handled separately,
 	 * since they don't require any conversion.
 	 */
@@ -60,11 +63,12 @@ convert_arg_to_entry (const char *set_value, ExifEntry *e, ExifByteOrder o, Exif
                 }
 		if (e->tag == EXIF_TAG_USER_COMMENT) {
 			/* assume ASCII charset */
+			/* TODO: get this from the current locale */
 			memcpy ((char *) e->data, "ASCII\0\0\0", 8);
-			memcpy ((char *) e->data + 8, (char *) set_value, 
+			memcpy ((char *) e->data + 8, set_value, 
 				strlen (set_value));
 		} else
-			strcpy ((char *) e->data, (char *) set_value);
+			strcpy ((char *) e->data, set_value);
                 return;
 	}
 
@@ -78,28 +82,23 @@ convert_arg_to_entry (const char *set_value, ExifEntry *e, ExifByteOrder o, Exif
 		exit (1);
 	}
 
-        value_p = (char*) set_value;
+	/* Copy the string so we can modify it */
+	buf = strdup(set_value);
+	if (!buf) exit(1);
+	value_p = strtok(buf, SEP);
 	numcomponents = e->components;
 	for (i = 0; i < numcomponents; ++i) {
-                const char *begin, *end;
-                unsigned char *buf, s;
-		static const char comp_separ = ' ';
+		unsigned char s;
 
-                begin = value_p;
-		value_p = strchr (begin, comp_separ);
 		if (!value_p) {
-                        if (i != numcomponents - 1) {
-                                fprintf (stderr, _("Too few components "
-						   "specified!"));
-				fputc ('\n', stderr);
+				fprintf (stderr, _("Too few components specified "
+					  "(need %d, found %d)\n"), numcomponents, i);
 				exit (1);
-                        }
-                        end = begin + strlen (begin);
-                } else end = value_p++;
-
-                buf = malloc ((end - begin + 1) * sizeof (char));
-                strncpy ((char *) buf, (char *) begin, end - begin);
-                buf[end - begin] = '\0';
+		}
+		if (!isdigit(*value_p) && (*value_p != '+') && (*value_p != '-')) {
+				fprintf (stderr, _("Numeric value expected\n"));
+				exit (1);
+		}
 
 		s = exif_format_get_size (e->format);
 		switch (e->format) {
@@ -108,10 +107,10 @@ convert_arg_to_entry (const char *set_value, ExifEntry *e, ExifByteOrder o, Exif
 				"Please contact <%s>."), PACKAGE_BUGREPORT);
 			break;
 		case EXIF_FORMAT_SHORT:
-			exif_set_short (e->data + (s * i), o, atoi ((char *) buf));
+			exif_set_short (e->data + (s * i), o, atoi (value_p));
 			break;
 		case EXIF_FORMAT_SSHORT:
-			exif_set_sshort (e->data + (s * i), o, atoi ((char *) buf));
+			exif_set_sshort (e->data + (s * i), o, atoi (value_p));
 			break;
 		case EXIF_FORMAT_RATIONAL:
 			/*
@@ -122,7 +121,7 @@ convert_arg_to_entry (const char *set_value, ExifEntry *e, ExifByteOrder o, Exif
 			s /= 2;
 			/* Fall through to LONG handler */
 		case EXIF_FORMAT_LONG:
-			exif_set_long (e->data + (s * i), o, atol ((char *) buf));
+			exif_set_long (e->data + (s * i), o, atol (value_p));
 			break;
 		case EXIF_FORMAT_SRATIONAL:
 			/*
@@ -133,23 +132,24 @@ convert_arg_to_entry (const char *set_value, ExifEntry *e, ExifByteOrder o, Exif
 			s /= 2;
 			/* Fall through to SLONG handler */
 		case EXIF_FORMAT_SLONG:
-			exif_set_slong (e->data + (s * i), o, atol ((char *) buf));
+			exif_set_slong (e->data + (s * i), o, atol (value_p));
 			break;
 		case EXIF_FORMAT_BYTE:
 		case EXIF_FORMAT_SBYTE:
-			e->data[s * i] = atoi ((char *) buf);
+		case EXIF_FORMAT_UNDEFINED: /* treat as byte array */
+			e->data[s * i] = atoi (value_p);
 			break;
 		case EXIF_FORMAT_FLOAT:
 		case EXIF_FORMAT_DOUBLE:
-		case EXIF_FORMAT_UNDEFINED:
 		default:
 			fprintf (stderr, _("Not yet implemented!"));
 			fputc ('\n', stderr);
 			exit (1);
 		}
-		free (buf);
+		value_p = strtok(NULL, SEP);
 	}
-	if (value_p && *value_p) {
+	free(buf);
+	if (value_p) {
 		fprintf (stderr, _("Warning; Too many components specified!"));
 		fputc ('\n', stderr);
 	}
@@ -225,6 +225,8 @@ action_create_value (ExifData *ed, ExifLog *log, ExifTag tag, ExifIfd ifd)
 	    e = exif_entry_new ();
 	    exif_content_add_entry (ed->ifd[ifd], e);
 	    exif_entry_initialize (e, tag);
+	    /* The entry has been added to the IFD, so we can unref it */
+	    exif_entry_unref(e);
 	}
 	return e;
 }
@@ -372,7 +374,7 @@ action_tag_table (ExifData *ed, ExifParams p)
 #define ENTRY_NOT_FOUND "   -   "
 
 	snprintf (txt, sizeof (txt) - 1, _("EXIF tags in '%s':"), p.fin);
-	fieldwidth = width = p.width - 42;
+	fieldwidth = width = p.width - 36;
 	bytes = exif_mbstrlen(txt, &width);
 	printf ("%.*s%*s", bytes, txt, fieldwidth-width, "");
 
@@ -397,9 +399,9 @@ action_tag_table (ExifData *ed, ExifParams p)
 		if (!name)
 			continue;
 
-		fieldwidth = width = p.width - 38 - 13;
+		fieldwidth = width = p.width - 43;
 		bytes = exif_mbstrlen(C(name), &width);
-		printf ("  0x%04x %.*s%*s",
+		printf ("0x%04x %.*s%*s",
 			tag, bytes, C(name), fieldwidth-width, "");
 		for (i = (ExifIfd)0; i < EXIF_IFD_COUNT; i++)
 			if (exif_content_get_entry (ed->ifd[i], tag))
@@ -472,14 +474,16 @@ action_mnote_list (ExifData *ed, ExifParams p)
 	}
 
 	c = exif_mnote_data_count (n);
-	switch (c) {
-	case 0:
-		printf (_("MakerNote does not contain any value.\n"));
-		break;
-	default:
-		printf (ngettext("MakerNote contains %i value:\n",
-			         "MakerNote contains %i values:\n",
-			 	 c), c);
+	if (!p.machine_readable) {
+		switch (c) {
+		case 0:
+			printf (_("MakerNote does not contain any value.\n"));
+			break;
+		default:
+			printf (ngettext("MakerNote contains %i value:\n",
+					 "MakerNote contains %i values:\n",
+					 c), c);
+		}
 	}
 	for (i = 0; i < c; i++) {
 	        if (p.use_ids) {
@@ -487,13 +491,16 @@ action_mnote_list (ExifData *ed, ExifParams p)
 			sprintf(b1,"0x%04x",id);
 		} else {
 			s = C (exif_mnote_data_get_title (n, i));
-			strncpy (b1, s && *s ? s : _("Unknown tag"), TAG_VALUE_BUF);
+			strncpy (b1, s && *s ? s : _("Unknown Tag"), TAG_VALUE_BUF);
 			b1[sizeof(b1)-1] = 0;
 		}
-		fieldwidth = width = p.use_ids ? 6 : 20;
-		bytes = exif_mbstrlen(b1, &width);
-                printf ("%.*s%*s", bytes, b1, fieldwidth-width, "");
-		fputc ('|', stdout);
+		if (p.machine_readable) {
+			printf ("%s\t", b1);
+		} else {
+			fieldwidth = width = p.use_ids ? 6 : 20;
+			bytes = exif_mbstrlen(b1, &width);
+			printf ("%.*s%*s|", bytes, b1, fieldwidth-width, "");
+		}
 
 		s = C (exif_mnote_data_get_value (n, i, b, TAG_VALUE_BUF));
 		strncpy (b2, s ? s : _("Unknown value"), TAG_VALUE_BUF);
